/*
Info: Owner draw ListBox for colored strings. ItemData is COLORREF value.
Compile: lcc -ansic -O -unused clistbox.c
         lcclnk -s -nolibc  -subsystem windows clistbox.obj
or using wededit: enable WinMain and remove GetCurrentProcess() command.
*/

#include <windows.h>

LRESULT CALLBACK MainWndProc( HWND, UINT, WPARAM, LPARAM );
HWND hwndList;

//int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, INT nCmdShow)
int main(void)
{
    MSG msg;
    HWND hwnd;
    WNDCLASS wc;
    HINSTANCE hInstance;

    hInstance = GetCurrentProcess();

    memset(&wc,0,sizeof(WNDCLASS));
    wc.style = CS_HREDRAW|CS_VREDRAW |CS_DBLCLKS ;
    wc.lpfnWndProc = (WNDPROC)MainWndProc;
    wc.hInstance = hInstance;
    wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
    wc.lpszClassName = "TestApp";
    wc.lpszMenuName = NULL;
    wc.hCursor = LoadCursor(NULL,IDC_ARROW);
    wc.hIcon = LoadIcon(NULL,IDI_APPLICATION);
    if (!RegisterClass(&wc))
        return 0;

    hwnd = CreateWindowEx(WS_EX_TOOLWINDOW, "TestApp","TestApp - clistbox", WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, CW_USEDEFAULT,150, 150 ,
        NULL, NULL, hInstance, NULL);
    if(hwnd == NULL)
        return 0;

    hwndList = CreateWindowEx(0, TEXT("LISTBOX"), TEXT(""),
        WS_VISIBLE | LBS_HASSTRINGS | WS_CHILD | LBS_OWNERDRAWFIXED,
        0, 0, 0, 0, hwnd, (HMENU) 100, hInstance, NULL);
    if(hwndList == NULL)
        return 0;

    SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM) TEXT("Colors") );
    SendMessage(hwndList, LB_SETITEMDATA, 0, (LPARAM) RGB(255,0,0) );

    SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM) TEXT("Colors") );
    SendMessage(hwndList, LB_SETITEMDATA, 1, (LPARAM) RGB(0,102,0) );

    SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM) TEXT("Colors") );
    SendMessage(hwndList, LB_SETITEMDATA, 2, (LPARAM) RGB(0,0,255) );

    SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM) TEXT("Colors") );
    SendMessage(hwndList, LB_SETITEMDATA, 3, (LPARAM) RGB(0,0,153) );

    SendMessage(hwndList, LB_ADDSTRING, 0, (LPARAM) TEXT("Colors") );
    SendMessage(hwndList, LB_SETITEMDATA, 4, (LPARAM) RGB(255,153,0) );

    ShowWindow(hwnd,SW_SHOW);

    while (GetMessage(&msg,NULL,0,0))
    {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }
    return msg.wParam;
}

LRESULT CALLBACK MainWndProc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
    LPDRAWITEMSTRUCT lpdis;
    LPMEASUREITEMSTRUCT lpmis;
    TCHAR tchBuffer[256];
    TEXTMETRIC tm;
    INT y;
    INT x;
    COLORREF clrForeground;
    COLORREF clrBackground;
    HANDLE hBrush;
    RECT rc;

    switch (msg) {

    // We can set vertical and horizontal spaces using this message.
    // But becose we don't need eny extra space we just return true.
    case WM_MEASUREITEM:
    return TRUE;

    case WM_DRAWITEM:
        lpdis = (LPDRAWITEMSTRUCT) lParam;

        if( lpdis->itemState == ODS_FOCUS)
        {
            DrawFocusRect(lpdis->hDC, &lpdis->rcItem);
            return TRUE;
        }

        // If there are no list box items, skip this message.
        if (lpdis->itemID == -1)
            break;

        // The colors depend on whether the item is selected.
        // When item or what color value item have
        if(lpdis->itemState & ODS_SELECTED)
            clrForeground = SetTextColor(lpdis->hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
        else
            clrForeground = SetTextColor(lpdis->hDC, lpdis->itemData);

        clrBackground = SetBkColor(lpdis->hDC,
            GetSysColor(lpdis->itemState & ODS_SELECTED ?
            COLOR_HIGHLIGHT : COLOR_WINDOW));

        // Calculate the vertical and horizontal position.
        GetTextMetrics(lpdis->hDC, &tm);
        y = (lpdis->rcItem.bottom + lpdis->rcItem.top - tm.tmHeight) / 2;
        x = LOWORD(GetDialogBaseUnits()) / 4;

        // Get and display the text for the list item.
        SendMessage(lpdis->hwndItem, LB_GETTEXT, lpdis->itemID, (LPARAM) tchBuffer);

        // If item is selected then background color needed to change
        // becose these are system brushs we don't need to free them.
        if (lpdis->itemState & ODS_SELECTED)
            hBrush = GetSysColorBrush(COLOR_HIGHLIGHT);
        else
            hBrush = GetSysColorBrush(COLOR_WINDOW);

        SendMessage(lpdis->hwndItem, LB_GETITEMRECT , lpdis->itemID,(LPARAM) &rc);
        FillRect(lpdis->hDC, &rc, hBrush);

        TextOut(lpdis->hDC, x, y, tchBuffer, strlen(tchBuffer));

        // If the item has the focus, draw focus rectangle.
        if (lpdis->itemState & ODS_SELECTED)
            DrawFocusRect(lpdis->hDC, &lpdis->rcItem);

        // Restore the previous colors.
        SetTextColor(lpdis->hDC, clrForeground);
        SetBkColor(lpdis->hDC, clrBackground);
        break;
    break;

    case WM_SETFOCUS:
        SetFocus(hwndList);
    break;


    case WM_SIZE:
        MoveWindow(hwndList, 0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
    break;

    case WM_DESTROY:
        PostQuitMessage(0);
        break;

    default:
        return DefWindowProc(hwnd,msg,wParam,lParam);
    }
    return 0;
}

